SystemOrganization addCategory: #'PersistentUnionFind-Tests'!
SystemOrganization addCategory: #PersistentUnionFind!

Object subclass: #ArrayPart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

ArrayPart subclass: #Arr
	instanceVariableNames: 'array'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

Arr class
	instanceVariableNames: 'array'!

!Arr commentStamp: 'fbs 12/28/2011 11:12' prior: 0!
An Arr is a wrapper for an array - really, a SequenceableCollection.!

Arr class
	instanceVariableNames: 'array'!

!Arr class methodsFor: 'instance creation' stamp: 'fbs 12/28/2011 11:45'!
wrapping: aSequenceableCollection
	^ self new wrapping: aSequenceableCollection.! !

!Arr methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:53'!
array
	^ array! !

!Arr methodsFor: 'converting' stamp: 'fbs 12/28/2011 12:38'!
asArray
	^ array copy.! !

!Arr methodsFor: 'printing' stamp: 'fbs 12/28/2011 15:59'!
printOn: aStream
	aStream
		nextPutAll: 'Arr(';
		nextPutAll: array printString;
		nextPutAll: ')'.! !

!Arr methodsFor: 'private' stamp: 'fbs 12/30/2011 08:13'!
wrapping: aSequenceableCollection
	| coll |
	coll := aSequenceableCollection copy.
	array := coll isDictionary
		ifTrue: [DictionaryWithIdentity wrapping: coll]
		ifFalse: [coll].! !

!ArrayPart methodsFor: 'converting' stamp: 'fbs 12/29/2011 11:17'!
asArray
	self subclassResponsibility.! !

!ArrayPart methodsFor: 'testing' stamp: 'fbs 12/29/2011 11:16'!
isDiff
	^ false.! !

ArrayPart subclass: #Diff
	instanceVariableNames: 'index value baseArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!Diff commentStamp: 'fbs 12/29/2011 10:44' prior: 0!
A Diff is a difference record. An instance is always relative to some Arr a, and means "like a, only with this element replaced with that value."

In particular, note that baseArray is a MutableCell referencing another PersistentArray.!

!Diff class methodsFor: 'instance creation' stamp: 'fbs 12/28/2011 11:39'!
index: anInteger value: anObject in: aPersistentArray
	^ self new index: anInteger value: anObject in: aPersistentArray.! !

!Diff methodsFor: 'converting' stamp: 'fbs 12/28/2011 12:45'!
asArray
	| arr |
	arr := baseArray asArray.
	arr at: index put: value.
	^ arr.! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/29/2011 11:08'!
baseArray
	^ baseArray.! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:34'!
index
	^ index.! !

!Diff methodsFor: 'private' stamp: 'fbs 12/28/2011 11:40'!
index: anInteger value: anObject in: aPersistentArray
	index := anInteger.
	value := anObject.
	baseArray := aPersistentArray.! !

!Diff methodsFor: 'testing' stamp: 'fbs 12/28/2011 11:27'!
isDiff
	^ true.! !

!Diff methodsFor: 'printing' stamp: 'fbs 12/29/2011 21:19'!
printOn: aStream
	aStream
		nextPutAll: 'Diff(';
		nextPutAll: index printString;
		nextPutAll: ', ';
		nextPutAll: value printString;
		nextPutAll: ', ';
		nextPutAll: baseArray printString;
		nextPutAll: ')'.
	! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:34'!
value
	^ value.! !

Object subclass: #DictionaryWithIdentity
	instanceVariableNames: 'dict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!DictionaryWithIdentity class methodsFor: 'instance creation' stamp: 'fbs 12/30/2011 08:03'!
wrapping: aDictionary
	^ self new wrapping: aDictionary.! !

!DictionaryWithIdentity methodsFor: 'accessing' stamp: 'fbs 12/30/2011 08:10'!
at: key
	^ dict at: key ifAbsent: [key].! !

!DictionaryWithIdentity methodsFor: 'accessing' stamp: 'fbs 12/30/2011 08:09'!
at: key put: value
	dict at: key put: value! !

!DictionaryWithIdentity methodsFor: 'printing' stamp: 'fbs 12/30/2011 22:39'!
printOn: aStream
	aStream
		nextPutAll: self class name.
	dict printElementsOn: aStream.	! !

!DictionaryWithIdentity methodsFor: 'private' stamp: 'fbs 12/30/2011 08:02'!
wrapping: aDictionary
	dict := aDictionary.! !

Object subclass: #PersistentUnionFind
	instanceVariableNames: 'arrayType ranks parents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!PersistentUnionFind commentStamp: 'fbs 12/28/2011 11:07' prior: 0!
A union-find data structure maintains a _partition_ of objects in some structure. That is, it implicitly defines an equivalence relation between the nodes of some structure.

The structure provides two methods - _find_ returns the representative of some class of nodes, and _union_ merges two classes of nodes.

* ranks is an array of the ranks (sizes) of the various equivalence classes.
* parents is an array of the representative terms for each of the equivalence classes.!

!PersistentUnionFind class methodsFor: 'instance creation' stamp: 'fbs 12/29/2011 17:46'!
usingArrayType: aClass partitioning: someObjects
	^ self new usingArrayType: aClass partitioning: someObjects.! !

!PersistentUnionFind methodsFor: 'accessing' stamp: 'fbs 12/29/2011 17:56'!
find: anObject
	"Return the representative element of anObject's equivalence class."
	| fAndCx |
	fAndCx := self findAux: anObject in: parents.
	parents := fAndCx first. "Path compression"
	^ fAndCx second.! !

!PersistentUnionFind methodsFor: 'private' stamp: 'fbs 12/29/2011 17:36'!
findAux: anObject in: anArray
	| representative pair |
	representative := anArray at: anObject.
	^ (representative = anObject)
		ifTrue: [{anArray. anObject.}]
		ifFalse: [ | f |
			pair := self findAux: representative in: anArray.
			f := anArray at: anObject put: representative.
			{f. representative.}]! !

!PersistentUnionFind methodsFor: 'private' stamp: 'fbs 12/29/2011 20:29'!
ranks: anArray parents: anotherArray
	ranks := anArray.
	parents := anotherArray.! !

!PersistentUnionFind methodsFor: 'as yet unclassified' stamp: 'fbs 12/30/2011 22:23'!
union: anObject with: anotherObject
	| cx cy rx ry |
	cx := self find: anObject.
	cy := self find: anotherObject.
	(cx = cy) ifTrue: [^ self].
	
	rx := ranks at: cx.
	ry := ranks at: cy.
	
	(rx > ry)
		ifTrue: [
			^ self class new
				ranks: (ranks at: cy put: ry + rx)
				parents: (parents at: cy put: cx)].
		
	(ry < rx)
		ifTrue: [
			^ self class new
				ranks: (ranks at: cx put: ry + rx)
				parents: (parents at: cx put: cy)].

	^ self class new
		ranks: (ranks at: cx put: rx + 1)
		parents: (parents at: cy put: cx).! !

!PersistentUnionFind methodsFor: 'private' stamp: 'fbs 12/29/2011 17:46'!
usingArrayType: aClass partitioning: someObjects
	arrayType := aClass.
	ranks := arrayType initially: (someObjects collect: [:unused| 1]).
	parents := arrayType initially: someObjects.! !

Object subclass: #Ref
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!Ref commentStamp: 'fbs 12/29/2011 13:59' prior: 0!
I wrap some other object, acting something like a pointer. Thus, things may have an immutable pointer to me, while having my value be mutable: an immutable reference to a mutable thing.!

Ref subclass: #PersistentCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!PersistentCollection commentStamp: 'fbs 12/29/2011 21:27' prior: 0!
A PersistentArray is a reference to an array-like structure that permits one fast access to the _most recent version_ of the array, but also (slower) access to _previous versions_ of the array.

#at: takes an index - an integer - and returns the value associated with that index.
#at:put: takes an index and an object and associates the latter with the former. It returns a new PersistentArray (and, as a side effect, 'reroots' the backing Arr and Diff structures).

An Arr wraps a SequenceableCollection. A Diff describes how to get a new array from an Arr - "like this Arr, but at index i put value v". Rerooting turns a base Arr - the original array - and a sequence of Diffs into an Arr - the current version - into a sequence of Diffs - how to recalculate a previous version - and the current version, an Arr. In other words, rerooting turns pointers into backpointers.

"Rerooting" permits O(1) access to arbitrary elements, and O(n) access to older versions of the array, where n is the number of versions.!

!PersistentCollection class methodsFor: 'instance creation' stamp: 'fbs 12/29/2011 11:38'!
initially: anArray
	^ self new wrapping: (Arr wrapping: anArray).! !

!PersistentCollection methodsFor: 'converting' stamp: 'fbs 12/29/2011 11:18'!
asArray
	^ self value asArray.! !

!PersistentCollection methodsFor: 'accessing' stamp: 'fbs 12/29/2011 21:06'!
at: anInteger
	| t |
	t := value.
	^ t isDiff
		ifFalse: [t array at: anInteger]
		ifTrue: [ | t1 |
			self reroot.
			t1 := value.
			t1 isDiff
				ifTrue: [Error signal: 'Reroot returned a Diff']
				ifFalse: [t1 array at: anInteger]]! !

!PersistentCollection methodsFor: 'accessing' stamp: 'fbs 12/30/2011 08:17'!
at: anInteger put: anObject
	| t |
	self reroot.
	((self at: anInteger) = anObject)
		ifTrue: [^ self].
	
	t := value.
	^ t isDiff
		ifTrue: [Error signal: 'Reroot returns a Diff']
		ifFalse: [ | t1 |
			t1 := value.
			t1 isDiff
				ifFalse: [| old res |
					old := t1 array at: anInteger.
					t1 array at: anInteger put: anObject.
					res := self class wrapping: t1.
					self value: (Diff index: anInteger value: old in: res).
					res]
				ifTrue: [self class wrapping:
							(Diff
								index: anInteger
								value: anObject
								in: self)]].! !

!PersistentCollection methodsFor: 'updating' stamp: 'fbs 12/29/2011 13:52'!
reroot
	| t t1 |
	t := value.
	t isDiff ifFalse: [^ self].
	
	t baseArray reroot.
	t1 := t baseArray value.
	t1 isDiff
		ifFalse: [ | v1 |
			v1 := t baseArray at: t index.
			t baseArray at: t index put: t value.
			self value: t1.
			t baseArray value: (Diff index: t index value: v1 in: self)]
		ifTrue: [Error signal: 'Reroot returned a Diff']! !

!Ref class methodsFor: 'instance creation' stamp: 'fbs 12/29/2011 11:13'!
wrapping: anObject
	^ self new wrapping: anObject.! !

!Ref methodsFor: 'printing' stamp: 'fbs 12/29/2011 11:20'!
printOn: aStream
	aStream nextPutAll: ('Ref({1})' format: {value printString})! !

!Ref methodsFor: 'accessing' stamp: 'fbs 12/29/2011 10:38'!
value
	^ value.! !

!Ref methodsFor: 'accessing' stamp: 'fbs 12/29/2011 11:13'!
value: anObject
	value := anObject.! !

!Ref methodsFor: 'private' stamp: 'fbs 12/29/2011 11:14'!
wrapping: anObject
	value := anObject.! !

TestCase subclass: #ArrTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!ArrTest methodsFor: 'testing - converting' stamp: 'fbs 12/28/2011 12:38'!
testAsArrayReturnsArray
	self assert: #(1 2 3) equals: (Arr wrapping: #(1 2 3)) asArray.! !

TestCase subclass: #DictionaryWithIdentityTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!DictionaryWithIdentityTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:07'!
testAtForKnownKeyReturnsMappedValue
	| d |
	d := DictionaryWithIdentity wrapping: (Dictionary newFromPairs: #(1 2)).
	self assert: 2 equals: (d at: 1).! !

!DictionaryWithIdentityTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:10'!
testAtForUnknownKeyReturnsKey
	| d |
	d := DictionaryWithIdentity wrapping: Dictionary new.
	self assert: 1 equals: (d at: 1).
	self assert: 2 equals: (d at: 2).! !

!DictionaryWithIdentityTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:08'!
testAtPutStoresMappedValue
	| d |
	d := DictionaryWithIdentity wrapping: Dictionary new.
	d at: 1 put: 2.
	self assert: 2 equals: (d at: 1).! !

!DictionaryWithIdentityTest methodsFor: 'testing - instance creation' stamp: 'fbs 12/30/2011 08:01'!
testCanWrapNormalDictionary
	DictionaryWithIdentity wrapping: Dictionary new.! !

TestCase subclass: #DiffTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

TestCase subclass: #PersistentCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:17'!
testAtOnChangedArrayReturnsAssociatedValue
	| arr arr2 |
	arr := PersistentCollection initially: #(3 2 1).
	arr2 := arr at: 2 put: 99.
	self assert: 99 equals: (arr2 at: 2).! !

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:17'!
testAtOnNewArrayReturnsAssociatedValue
	| arr |
	arr := PersistentCollection initially: #(3 2 1).
	self assert: 2 equals: (arr at: 2).! !

!PersistentCollectionTest methodsFor: 'testing - maps' stamp: 'fbs 12/30/2011 08:19'!
testAtOnUnknownValueWorksLikeIdentity
	| map |
	map := PersistentCollection initially: Dictionary new.
	self assert: 1 equals: (map at: 1).! !

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:18'!
testAtPutLeavesWrappedArrayUnchanged
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentCollection initially: array.
	persistentArray := persistentArray at: 1 put: 4.

	self assert: #(3 2 1) equals: array.! !

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:18'!
testAtPutOfSameValueApparentlyDoesNothing
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentCollection initially: array.
	persistentArray := persistentArray at: 1 put: 4.
	persistentArray := persistentArray at: 1 put: 4.

	self assert: #(4 2 1) equals: persistentArray asArray.! !

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:18'!
testAtPutReturnsaPersistentArray
	| array persistentArray persistentArray2 |
	array := #(3 2 1).
	persistentArray := PersistentCollection initially: array.
	persistentArray2 := persistentArray at: 1 put: 4.
	self assert: #(4 2 1) equals: persistentArray2 asArray.! !

!PersistentCollectionTest methodsFor: 'testing - maps' stamp: 'fbs 12/30/2011 08:18'!
testAtPutWorksLikeaMap
	| map |
	map := PersistentCollection initially: Dictionary new.
	map := map at: 1 put: 4.
	self assert: 4 equals: (map at: 1).! !

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:18'!
testAtPutWorksOnImmediateReferences
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentCollection initially: array.
	persistentArray := persistentArray at: 1 put: 4.
	self assert: 4 equals: (persistentArray at: 1).! !

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/29/2011 11:40'!
testAtWorksOnImmediateReferences
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentArray initially: array.
	self assert: 3 equals: (persistentArray at: 1).
	self assert: 2 equals: (persistentArray at: 2).
	self assert: 1 equals: (persistentArray at: 3).! !

!PersistentCollectionTest methodsFor: 'testing - accessing' stamp: 'fbs 12/30/2011 08:19'!
testPersistentArraysPersist
	| arr arr2 arr3 arr4 |
	arr := PersistentCollection initially: #(1 2 3).
	arr2 := arr at: 1 put: 4.
	arr3 := arr2 at: 2 put: 5.
	arr4 := arr2 at: 3 put: 6.
	
	"A linear chain of changes."
	self assert: #(1 2 3) equals: arr asArray.
	self assert: #(4 2 3) equals: arr2 asArray.
	self assert: #(4 2 6) equals: arr4 asArray.
	
	"The alternative branch of changes."
	self assert: #(4 5 3) equals: arr3 asArray.! !

!PersistentCollectionTest methodsFor: 'testing - printing' stamp: 'fbs 12/30/2011 08:19'!
testPrintOnPrintsSensiblyForNew
	| a |
	a := PersistentCollection initially: #(1 2 3).
	self assert: 'Ref(Arr(#(1 2 3)))' equals: a printString.! !

!PersistentCollectionTest methodsFor: 'testing - printing' stamp: 'fbs 12/30/2011 08:19'!
testPrintOnPrintsSensiblyForUpdatedArrays
	| d |
	d := Diff index: 1 value: 4 in: (PersistentCollection initially: #(1 2 3)).
	self assert: 'Diff(1, 4, Ref(Arr(#(1 2 3))))' equals: d printString.! !

!PersistentCollectionTest methodsFor: 'testing - updating' stamp: 'fbs 12/30/2011 08:19'!
testRerootLeavesArraySeeminglyUnchanged
	| arr arr2 arr3 |
	arr := PersistentCollection initially: #(1 2 3).
	arr2 := arr at: 3 put: 4.
	arr3 := arr2 reroot.
	
	self assert: #(1 2 4) equals: arr3 asArray! !

TestCase subclass: #PersistentUnionFindTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!PersistentUnionFindTest methodsFor: 'testing' stamp: 'fbs 12/30/2011 08:19'!
testInitialStateHasNoUnification
	| partition data |
	data := #(1 2 3 4).
	partition := PersistentUnionFind usingArrayType: PersistentCollection partitioning: data.
	self assert: data first equals: (partition find: data first).
	self assert: data second equals: (partition find: data second).
	self assert: data third equals: (partition find: data third).
	self assert: data fourth equals: (partition find: data fourth).! !

!PersistentUnionFindTest methodsFor: 'testing - union' stamp: 'fbs 12/30/2011 08:19'!
testUnionLeavesOtherEquivalenceClassesUnchanged
	| partition p2 data |
	data := #(1 2 3 4).
	partition := PersistentUnionFind usingArrayType: PersistentCollection partitioning: data.
	p2 := partition union: 1 with: 3.

	self assert: 3 equals: (partition find: 3).
	self assert: 4 equals: (partition find: 4).! !

!PersistentUnionFindTest methodsFor: 'testing - union' stamp: 'fbs 12/30/2011 08:19'!
testUnionMergesTwoEquivalenceClasses
	| partition p2 data |
	data := #(1 2 3 4).
	partition := PersistentUnionFind usingArrayType: PersistentCollection partitioning: data.

	p2 := partition union: 1 with: 3.
	self assert: (p2 find: 1) equals: (p2 find: 3).! !

!PersistentUnionFindTest methodsFor: 'testing - union' stamp: 'fbs 12/30/2011 08:19'!
testUnionsDontAffectPreviousVersions
	| partition p2 p3 p4 data |
	data := #(1 2 3 4).
	partition := PersistentUnionFind usingArrayType: PersistentCollection partitioning: data.
	p2 := partition union: 1 with: 3.
	p3 := partition union: 2 with: 4.
	p4 := partition union: 1 with: 2.
	self assert: (p2 find: 1) equals: (p2 find: 3).
	self deny: (p2 find: 1) = (p2 find: 2).! !
